pub fn sycamore_circuit<R>(qubits: usize, depth: usize, rng: &mut R) -> Circuitwhere
R: Rng,Expand description
Creates a circuit based on the Sycamore circuit scheme.
The depth is the number of rounds, where one round consists of a layer of
single-qubit gates followed by a layer of two-qubit gates. The circuit is
initialized in the |0> state.
For more details on the circuit, see https://arxiv.org/abs/1910.11333.