LeafTensor

Struct LeafTensor 

pub struct LeafTensor(/* private fields */);
A single leaf tensor.

Implementations

Source

impl LeafTensor

pub fn new_from_map( legs: Vec<EdgeIndex>, bond_dims_map: &FxHashMap<EdgeIndex, u64>, ) -> Self

Constructs a leaf tensor with the given edge ids and a mapping of edge ids to corresponding bond dimensions.

Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.legs(), &[1, 2, 3]);
assert_eq!(tensor.bond_dims(), &[2, 4, 6]);
pub fn new_from_const(legs: Vec<EdgeIndex>, bond_dim: u64) -> Self

Constructs a leaf tensor with the given edge ids and the same bond dimension for all edges.

Examples
let tensor = LeafTensor::new_from_const(vec![1, 2, 3], 2);
assert_eq!(tensor.legs(), &[1, 2, 3]);
assert_eq!(tensor.bond_dims(), &[2, 2, 2]);
pub fn as_tensor(&self) -> &Tensor

Returns a reference to this leaf tensor as a Tensor.

pub fn shallow_clone(&self) -> Self

Returns a new leaf tensor with the same legs and bond dimensions as this tensor, but without any data.

pub fn legs(&self) -> &Vec<EdgeIndex>

Returns edge ids of the tensor.

Examples
let tensor = LeafTensor::new_from_const(vec![1, 2, 3], 3);
assert_eq!(tensor.legs(), &[1, 2, 3]);
pub fn edges(&self) -> impl Iterator<Item = (EdgeIndex, u64)> + '_

Returns an iterator of tuples of leg ids and their corresponding bond size.

pub fn bond_dims(&self) -> &Vec<u64>

Getter for bond dimensions.

pub fn shape(&self) -> Result<Vec<usize>, TryFromIntError>

Returns the shape of tensor. This is the same as the bond dimensions, but as usize. The conversion can fail, hence a Result is returned.

pub fn dims(&self) -> usize

Returns the number of dimensions.

Examples
let bond_dims = FxHashMap::from_iter([(1, 4), (2, 6), (3, 2)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.dims(), 3);
pub fn size(&self) -> f64

Returns the number of elements. This is a f64 to avoid overflow in large tensors.

Examples
let bond_dims = FxHashMap::from_iter([(1, 5), (2, 15), (3, 8)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.size(), 600.0);
pub fn into_legs(self) -> Vec<EdgeIndex>

Converts this tensor into the leg ids and bond dimensions it is made of.

pub fn into_data(self) -> TensorData

Converts this tensor into the data it contains.

pub fn into_inner(self) -> (Vec<EdgeIndex>, Vec<u64>, TensorData)

Converts this tensor into the leg ids, bond dimensions and data it is made of.

pub fn tensor_data(&self) -> &TensorData

Getter for tensor data.

pub fn set_tensor_data(&mut self, tensordata: TensorData)

Setter for tensor data.

Examples
let mut tensor = LeafTensor::new_from_const(vec![0, 1], 2);
let tensordata = TensorData::Gate((String::from("x"), vec![], false));
tensor.set_tensor_data(tensordata);
pub fn difference(&self, other: &Self) -> Self

Returns the tensor with legs in self that are not in other.

Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let diff_tensor = &tensor1 - &tensor2;
assert_eq!(diff_tensor.legs(), &[1, 3]);
assert_eq!(diff_tensor.bond_dims(), &[2, 6]);
pub fn union(&self, other: &Self) -> Self

Returns the tensor with union of legs in both self and other.

Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let union_tensor = &tensor1 | &tensor2;
assert_eq!(union_tensor.legs(), &[1, 2, 3, 4, 5]);
assert_eq!(union_tensor.bond_dims(), &[2, 4, 6, 3, 9]);
pub fn intersection(&self, other: &Self) -> Self

Returns the tensor with intersection of legs in self and other.

Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let intersection_tensor = &tensor1 & &tensor2;
assert_eq!(intersection_tensor.legs(), &[2]);
assert_eq!(intersection_tensor.bond_dims(), &[4]);
pub fn symmetric_difference(&self, other: &Self) -> Self

Returns the tensor with symmetrical difference of legs in self and other.

Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let sym_dif_tensor = &tensor1 ^ &tensor2;
assert_eq!(sym_dif_tensor.legs(), &[1, 3, 4, 5]);
assert_eq!(sym_dif_tensor.bond_dims(), &[2, 6, 3, 9]);