pub struct Tensor(/* private fields */);Expand description
An abstract tensor. This can either be a CompositeTensor or a LeafTensor.
Implementations
Sourceimpl Tensor
impl Tensor
Sourcepub fn kind(&self) -> TensorType
pub fn kind(&self) -> TensorType
Returns the kind of this tensor.
Sourcepub fn is_leaf(&self) -> bool
pub fn is_leaf(&self) -> bool
Returns true if the tensor is a leaf tensor, without any nested tensors.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6)]);
let leaf: Tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims).into();
assert_eq!(leaf.is_leaf(), true);
let comp: Tensor = CompositeTensor::new(vec![leaf]).into();
assert_eq!(comp.is_leaf(), false);Sourcepub fn as_leaf(&self) -> Option<&LeafTensor>
pub fn as_leaf(&self) -> Option<&LeafTensor>
Returns a reference to this tensor as a LeafTensor if it is one.
Sourcepub fn into_leaf(self) -> Option<LeafTensor>
pub fn into_leaf(self) -> Option<LeafTensor>
Returns this tensor as a LeafTensor if it is one.
Sourcepub fn is_composite(&self) -> bool
pub fn is_composite(&self) -> bool
Returns true if the tensor is composite.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6)]);
let leaf: Tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims).into();
assert_eq!(leaf.is_composite(), false);
let comp: Tensor = CompositeTensor::new(vec![leaf]).into();
assert_eq!(comp.is_composite(), true);Sourcepub fn as_composite(&self) -> Option<&CompositeTensor>
pub fn as_composite(&self) -> Option<&CompositeTensor>
Returns a reference to this tensor as a CompositeTensor if it is one.
Sourcepub fn into_composite(self) -> Option<CompositeTensor>
pub fn into_composite(self) -> Option<CompositeTensor>
Returns this tensor as a CompositeTensor if it is one.