pub struct LeafTensor(/* private fields */);Expand description
A single leaf tensor.
Implementations
Sourceimpl LeafTensor
impl LeafTensor
Sourcepub fn new_from_map(
legs: Vec<EdgeIndex>,
bond_dims_map: &FxHashMap<EdgeIndex, u64>,
) -> Self
pub fn new_from_map( legs: Vec<EdgeIndex>, bond_dims_map: &FxHashMap<EdgeIndex, u64>, ) -> Self
Constructs a leaf tensor with the given edge ids and a mapping of edge ids to corresponding bond dimensions.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.legs(), &[1, 2, 3]);
assert_eq!(tensor.bond_dims(), &[2, 4, 6]);Sourcepub fn new_from_const(legs: Vec<EdgeIndex>, bond_dim: u64) -> Self
pub fn new_from_const(legs: Vec<EdgeIndex>, bond_dim: u64) -> Self
Constructs a leaf tensor with the given edge ids and the same bond dimension for all edges.
Examples
let tensor = LeafTensor::new_from_const(vec![1, 2, 3], 2);
assert_eq!(tensor.legs(), &[1, 2, 3]);
assert_eq!(tensor.bond_dims(), &[2, 2, 2]);Sourcepub fn shallow_clone(&self) -> Self
pub fn shallow_clone(&self) -> Self
Returns a new leaf tensor with the same legs and bond dimensions as this tensor, but without any data.
Sourcepub fn legs(&self) -> &Vec<EdgeIndex> ⓘ
pub fn legs(&self) -> &Vec<EdgeIndex> ⓘ
Returns edge ids of the tensor.
Examples
let tensor = LeafTensor::new_from_const(vec![1, 2, 3], 3);
assert_eq!(tensor.legs(), &[1, 2, 3]);Sourcepub fn edges(&self) -> impl Iterator<Item = (EdgeIndex, u64)> + '_
pub fn edges(&self) -> impl Iterator<Item = (EdgeIndex, u64)> + '_
Returns an iterator of tuples of leg ids and their corresponding bond size.
Sourcepub fn shape(&self) -> Result<Vec<usize>, TryFromIntError>
pub fn shape(&self) -> Result<Vec<usize>, TryFromIntError>
Returns the shape of tensor. This is the same as the bond dimensions, but as
usize. The conversion can fail, hence a Result is returned.
Sourcepub fn dims(&self) -> usize
pub fn dims(&self) -> usize
Returns the number of dimensions.
Examples
let bond_dims = FxHashMap::from_iter([(1, 4), (2, 6), (3, 2)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.dims(), 3);Sourcepub fn size(&self) -> f64
pub fn size(&self) -> f64
Returns the number of elements. This is a f64 to avoid overflow in large tensors.
Examples
let bond_dims = FxHashMap::from_iter([(1, 5), (2, 15), (3, 8)]);
let tensor = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
assert_eq!(tensor.size(), 600.0);Sourcepub fn into_legs(self) -> Vec<EdgeIndex> ⓘ
pub fn into_legs(self) -> Vec<EdgeIndex> ⓘ
Converts this tensor into the leg ids and bond dimensions it is made of.
Sourcepub fn into_data(self) -> TensorData
pub fn into_data(self) -> TensorData
Converts this tensor into the data it contains.
Sourcepub fn into_inner(self) -> (Vec<EdgeIndex>, Vec<u64>, TensorData)
pub fn into_inner(self) -> (Vec<EdgeIndex>, Vec<u64>, TensorData)
Converts this tensor into the leg ids, bond dimensions and data it is made of.
Sourcepub fn tensor_data(&self) -> &TensorData
pub fn tensor_data(&self) -> &TensorData
Getter for tensor data.
Sourcepub fn set_tensor_data(&mut self, tensordata: TensorData)
pub fn set_tensor_data(&mut self, tensordata: TensorData)
Setter for tensor data.
Examples
let mut tensor = LeafTensor::new_from_const(vec![0, 1], 2);
let tensordata = TensorData::Gate((String::from("x"), vec![], false));
tensor.set_tensor_data(tensordata);Sourcepub fn difference(&self, other: &Self) -> Self
pub fn difference(&self, other: &Self) -> Self
Returns the tensor with legs in self that are not in other.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let diff_tensor = &tensor1 - &tensor2;
assert_eq!(diff_tensor.legs(), &[1, 3]);
assert_eq!(diff_tensor.bond_dims(), &[2, 6]);Sourcepub fn union(&self, other: &Self) -> Self
pub fn union(&self, other: &Self) -> Self
Returns the tensor with union of legs in both self and other.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let union_tensor = &tensor1 | &tensor2;
assert_eq!(union_tensor.legs(), &[1, 2, 3, 4, 5]);
assert_eq!(union_tensor.bond_dims(), &[2, 4, 6, 3, 9]);Sourcepub fn intersection(&self, other: &Self) -> Self
pub fn intersection(&self, other: &Self) -> Self
Returns the tensor with intersection of legs in self and other.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let intersection_tensor = &tensor1 & &tensor2;
assert_eq!(intersection_tensor.legs(), &[2]);
assert_eq!(intersection_tensor.bond_dims(), &[4]);Sourcepub fn symmetric_difference(&self, other: &Self) -> Self
pub fn symmetric_difference(&self, other: &Self) -> Self
Returns the tensor with symmetrical difference of legs in self and other.
Examples
let bond_dims = FxHashMap::from_iter([(1, 2), (2, 4), (3, 6), (4, 3), (5, 9)]);
let tensor1 = LeafTensor::new_from_map(vec![1, 2, 3], &bond_dims);
let tensor2 = LeafTensor::new_from_map(vec![4, 2, 5], &bond_dims);
let sym_dif_tensor = &tensor1 ^ &tensor2;
assert_eq!(sym_dif_tensor.legs(), &[1, 3, 4, 5]);
assert_eq!(sym_dif_tensor.bond_dims(), &[2, 6, 3, 9]);Trait Implementations
Sourceimpl AbsDiffEq for LeafTensor
impl AbsDiffEq for LeafTensor
Sourcefn default_epsilon() -> Self::Epsilon
fn default_epsilon() -> Self::Epsilon
Sourcefn abs_diff_eq(&self, other: &Self, epsilon: Self::Epsilon) -> bool
fn abs_diff_eq(&self, other: &Self, epsilon: Self::Epsilon) -> bool
fn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
fn abs_diff_ne(&self, other: &Rhs, epsilon: Self::Epsilon) -> bool
AbsDiffEq::abs_diff_eq].Sourceimpl BitAnd for &LeafTensor
impl BitAnd for &LeafTensor
Sourceimpl BitOr for &LeafTensor
impl BitOr for &LeafTensor
Sourceimpl BitXor for &LeafTensor
impl BitXor for &LeafTensor
Sourceimpl BitXorAssign<&LeafTensor> for LeafTensor
impl BitXorAssign<&LeafTensor> for LeafTensor
Sourcefn bitxor_assign(&mut self, rhs: &Self)
fn bitxor_assign(&mut self, rhs: &Self)
^= operation. Read moreSourceimpl Clone for LeafTensor
impl Clone for LeafTensor
Sourcefn clone(&self) -> LeafTensor
fn clone(&self) -> LeafTensor
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for LeafTensor
impl Debug for LeafTensor
Sourceimpl Default for LeafTensor
impl Default for LeafTensor
Sourceimpl<'de> Deserialize<'de> for LeafTensor
impl<'de> Deserialize<'de> for LeafTensor
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Sourceimpl From<LeafTensor> for Tensor
impl From<LeafTensor> for Tensor
Sourcefn from(value: LeafTensor) -> Self
fn from(value: LeafTensor) -> Self
Sourceimpl PartialEq for LeafTensor
impl PartialEq for LeafTensor
Sourcefn eq(&self, other: &LeafTensor) -> bool
fn eq(&self, other: &LeafTensor) -> bool
self and other values to be equal, and is used by ==.Sourceimpl Serialize for LeafTensor
impl Serialize for LeafTensor
Sourceimpl Sub for &LeafTensor
impl Sub for &LeafTensor
Sourceimpl TransparentWrapper<TensorRepr> for LeafTensor
impl TransparentWrapper<TensorRepr> for LeafTensor
fn wrap_ref(s: &Inner) -> &Self
fn wrap_ref(s: &Inner) -> &Self
fn wrap_mut(s: &mut Inner) -> &mut Self
fn wrap_mut(s: &mut Inner) -> &mut Self
fn wrap_slice(s: &[Inner]) -> &[Self]where
Self: Sized,
fn wrap_slice(s: &[Inner]) -> &[Self]where
Self: Sized,
fn wrap_slice_mut(s: &mut [Inner]) -> &mut [Self]where
Self: Sized,
fn wrap_slice_mut(s: &mut [Inner]) -> &mut [Self]where
Self: Sized,
fn peel_ref(s: &Self) -> &Inner
fn peel_ref(s: &Self) -> &Inner
fn peel_mut(s: &mut Self) -> &mut Inner
fn peel_mut(s: &mut Self) -> &mut Inner
fn peel_slice(s: &[Self]) -> &[Inner]where
Self: Sized,
fn peel_slice(s: &[Self]) -> &[Inner]where
Self: Sized,
fn peel_slice_mut(s: &mut [Self]) -> &mut [Inner]where
Self: Sized,
fn peel_slice_mut(s: &mut [Self]) -> &mut [Inner]where
Self: Sized,
impl StructuralPartialEq for LeafTensor